/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_AutoReadLock.qpp
 
  Qore Programming Language
 
  Copyright (C) 2003 - 2023 David Nichols
 
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_AutoReadLock.h"
#include "qore/intern/QC_RWLock.h"
/* Qore class Qore::Thread::AutoReadLock */

qore_classid_t CID_AUTOREADLOCK;
QoreClass* QC_AUTOREADLOCK;

// AutoReadLock::constructor(RWLock rwl) {}
static void AutoReadLock_constructor_C6RWLock(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(rwl, RWLock, args, 0, CID_RWLOCK, "AutoReadLock::constructor()", "RWLock", xsink);
    if (*xsink)
        return;
# 73 "QC_AutoReadLock.qpp"
ReferenceHolder<QoreAutoReadLock> arwl(new QoreAutoReadLock(rwl, xsink), xsink);
   if (!*xsink)
      self->setPrivate(CID_AUTOREADLOCK, arwl.release());
}

// AutoReadLock::copy() {}
static void AutoReadLock_copy(QoreObject* self, QoreObject* old, QoreAutoReadLock* arwl, ExceptionSink* xsink) {
# 97 "QC_AutoReadLock.qpp"
xsink->raiseException("AUTOREADLOCK-COPY-ERROR", "objects of this class cannot be copied");
}

// AutoReadLock::destructor() {}
static void AutoReadLock_destructor(QoreObject* self, QoreAutoReadLock* arwl, ExceptionSink* xsink) {
# 88 "QC_AutoReadLock.qpp"
arwl->destructor(xsink);
   arwl->deref(xsink);
}

DLLLOCAL void preinitAutoReadLockClass() {
    QC_AUTOREADLOCK = new QoreClass("AutoReadLock", "::Qore::Thread::AutoReadLock", QDOM_THREAD_CLASS);
    CID_AUTOREADLOCK = QC_AUTOREADLOCK->getID();
    QC_AUTOREADLOCK->setSystem();
}

DLLLOCAL QoreClass* initAutoReadLockClass(QoreNamespace& ns) {
    if (!QC_AUTOREADLOCK)
        preinitAutoReadLockClass();

    // AutoReadLock::constructor(RWLock rwl) {}
    QC_AUTOREADLOCK->addConstructor(AutoReadLock_constructor_C6RWLock, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_RWLOCK->getTypeInfo(), QORE_PARAM_NO_ARG, "rwl");

    // AutoReadLock::copy() {}
    QC_AUTOREADLOCK->setCopy((q_copy_t)AutoReadLock_copy);

    // AutoReadLock::destructor() {}
    QC_AUTOREADLOCK->setDestructor((q_destructor_t)AutoReadLock_destructor);

    return QC_AUTOREADLOCK;
}
