/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Counter.cpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Counter.h"
/* Qore class Qore::Thread::Counter */

qore_classid_t CID_COUNTER;
QoreClass* QC_COUNTER;

// Counter::constructor(softint c = 0) {}
static void Counter_constructor_vi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 c = HARD_QORE_VALUE_INT(args, 0);
# 53 "QC_Counter.qpp"
if (c < 0) {
      xsink->raiseException("COUNTER-ERROR", "Counter::constructor() called with initial value " QLLD ", however value must be non-negative", c);
      return;
   }
   self->setPrivate(CID_COUNTER, new Counter(c));
}

// Counter::copy() {}
static void Counter_copy(QoreObject* self, QoreObject* old, Counter* c, ExceptionSink* xsink) {
# 83 "QC_Counter.qpp"
self->setPrivate(CID_COUNTER, new Counter(c->getCount()));
}

// int Counter::dec(){}
static QoreValue Counter_dec(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_Counter.qpp"
    return c->dec(xsink);
}

// Counter::destructor() {}
static void Counter_destructor(QoreObject* self, Counter* c, ExceptionSink* xsink) {
# 71 "QC_Counter.qpp"
c->destructor(xsink);
   c->deref(xsink);
}

// int Counter::getCount(){}
static QoreValue Counter_getCount(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 158 "QC_Counter.qpp"
    return c->getCount();
}

// int Counter::getWaiting(){}
static QoreValue Counter_getWaiting(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 170 "QC_Counter.qpp"
    return c->getWaiting();
}

// int Counter::inc(){}
static QoreValue Counter_inc(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 98 "QC_Counter.qpp"
    return c->inc();
}

// nothing Counter::waitForZero(){}
static QoreValue Counter_waitForZero(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 129 "QC_Counter.qpp"
    c->waitForZero(xsink);
    return QoreValue();
}

// int Counter::waitForZero(timeout timeout_ms){}
static QoreValue Counter_waitForZero_Vt(QoreObject* self, Counter* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 146 "QC_Counter.qpp"
    return c->waitForZero(xsink, timeout_ms);
}

DLLLOCAL void preinitCounterClass() {
    QC_COUNTER = new QoreClass("Counter", "::Qore::Thread::Counter", QDOM_THREAD_CLASS);
    CID_COUNTER = QC_COUNTER->getID();
    QC_COUNTER->setSystem();
}

DLLLOCAL QoreClass* initCounterClass(QoreNamespace& ns) {
    if (!QC_COUNTER)
        preinitCounterClass();

    // Counter::constructor(softint c = 0) {}
    QC_COUNTER->addConstructor(Counter_constructor_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "c");

    // Counter::copy() {}
    QC_COUNTER->setCopy((q_copy_t)Counter_copy);

    // int Counter::dec(){}
    QC_COUNTER->addMethod("dec", (q_method_n_t)Counter_dec, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // Counter::destructor() {}
    QC_COUNTER->setDestructor((q_destructor_t)Counter_destructor);

    // int Counter::getCount(){}
    QC_COUNTER->addMethod("getCount", (q_method_n_t)Counter_getCount, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Counter::getWaiting(){}
    QC_COUNTER->addMethod("getWaiting", (q_method_n_t)Counter_getWaiting, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Counter::inc(){}
    QC_COUNTER->addMethod("inc", (q_method_n_t)Counter_inc, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Counter::waitForZero(){}
    QC_COUNTER->addMethod("waitForZero", (q_method_n_t)Counter_waitForZero, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Counter::waitForZero(timeout timeout_ms){}
    QC_COUNTER->addMethod("waitForZero", (q_method_n_t)Counter_waitForZero_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    return QC_COUNTER;
}
