/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_DebugProgram.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_DebugProgram.h"
#include "qore/intern/QC_Program.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/AbstractStatement.h"
/* Qore class Qore::DebugProgram */

qore_classid_t CID_DEBUGPROGRAM;
QoreClass* QC_DEBUGPROGRAM;

// nothing DebugProgram::addProgram(ProgramControl pgm){}
static QoreValue DebugProgram_addProgram_C14ProgramControl(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAMCONTROL, "DebugProgram::addProgram()", "ProgramControl", xsink);
    if (*xsink)
        return 0;
# 161 "QC_DebugProgram.qpp"
    ReferenceHolder<QoreProgram> holder(pgm, xsink);
   dpgm->addProgram(pgm, xsink);
    return QoreValue();
}

// int DebugProgram::breakProgram(ProgramControl pgm){}
static QoreValue DebugProgram_breakProgram_C14ProgramControl(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAMCONTROL, "DebugProgram::breakProgram()", "ProgramControl", xsink);
    if (*xsink)
        return 0;
# 241 "QC_DebugProgram.qpp"
    ReferenceHolder<QoreProgram> holder(pgm, xsink);
   return dpgm->breakProgram(pgm);
}

// int DebugProgram::breakProgramThread(ProgramControl pgm, int tid){}
static QoreValue DebugProgram_breakProgramThread_C14ProgramControlVi(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAMCONTROL, "DebugProgram::breakProgramThread()", "ProgramControl", xsink);
    if (*xsink)
        return 0;
    int64 tid = HARD_QORE_VALUE_INT(args, 1);
# 232 "QC_DebugProgram.qpp"
    ReferenceHolder<QoreProgram> holder(pgm, xsink);
   return dpgm->breakProgramThread(pgm, tid);
}

// DebugProgram::constructor() {}
static void DebugProgram_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 120 "QC_DebugProgram.qpp"
self->setPrivate(CID_DEBUGPROGRAM, new QoreDebugProgramWithQoreObject(self));
}

// DebugProgram::copy() {}
static void DebugProgram_copy(QoreObject* self, QoreObject* old, QoreDebugProgramWithQoreObject* dpgm, ExceptionSink* xsink) {
# 128 "QC_DebugProgram.qpp"
xsink->raiseException("DEBUGPROGRAM-COPY-ERROR", "copying DebugProgram objects is unsupported");
}

// DebugProgram::destructor() {}
static void DebugProgram_destructor(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, ExceptionSink* xsink) {
# 136 "QC_DebugProgram.qpp"
dpgm->waitForTerminationAndClear(xsink);
   dpgm->deref(xsink);
}

// list DebugProgram::getAllPrograms(){}
static QoreValue DebugProgram_getAllPrograms(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 179 "QC_DebugProgram.qpp"
    return dpgm->getAllProgramObjects();
}

// int DebugProgram::getInterruptedCount(){}
static QoreValue DebugProgram_getInterruptedCount(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 153 "QC_DebugProgram.qpp"
    dpgm->getInterruptedCount();
    return QoreValue();
}

// nothing DebugProgram::removeProgram(ProgramControl pgm){}
static QoreValue DebugProgram_removeProgram_C14ProgramControl(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAMCONTROL, "DebugProgram::removeProgram()", "ProgramControl", xsink);
    if (*xsink)
        return 0;
# 170 "QC_DebugProgram.qpp"
    ReferenceHolder<QoreProgram> holder(pgm, xsink);
   dpgm->removeProgram(pgm);
    return QoreValue();
}

// nothing DebugProgram::waitForTerminationAndClear(){}
static QoreValue DebugProgram_waitForTerminationAndClear(QoreObject* self, QoreDebugProgramWithQoreObject* dpgm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 145 "QC_DebugProgram.qpp"
    dpgm->waitForTerminationAndClear(xsink);
    return QoreValue();
}

DLLLOCAL void preinitDebugProgramClass() {
    QC_DEBUGPROGRAM = new QoreClass("DebugProgram", "::Qore::DebugProgram", QDOM_DEBUGGER);
    CID_DEBUGPROGRAM = QC_DEBUGPROGRAM->getID();
    QC_DEBUGPROGRAM->setSystem();
}

DLLLOCAL QoreClass* initDebugProgramClass(QoreNamespace& ns) {
    if (!QC_DEBUGPROGRAM)
        preinitDebugProgramClass();

    // nothing DebugProgram::addProgram(ProgramControl pgm){}
    QC_DEBUGPROGRAM->addMethod("addProgram", (q_method_n_t)DebugProgram_addProgram_C14ProgramControl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm");

    // int DebugProgram::breakProgram(ProgramControl pgm){}
    QC_DEBUGPROGRAM->addMethod("breakProgram", (q_method_n_t)DebugProgram_breakProgram_C14ProgramControl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm");

    // int DebugProgram::breakProgramThread(ProgramControl pgm, int tid){}
    QC_DEBUGPROGRAM->addMethod("breakProgramThread", (q_method_n_t)DebugProgram_breakProgramThread_C14ProgramControlVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "tid");

    // DebugProgram::constructor() {}
    QC_DEBUGPROGRAM->addConstructor(DebugProgram_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // DebugProgram::copy() {}
    QC_DEBUGPROGRAM->setCopy((q_copy_t)DebugProgram_copy);

    // DebugProgram::destructor() {}
    QC_DEBUGPROGRAM->setDestructor((q_destructor_t)DebugProgram_destructor);

    // list DebugProgram::getAllPrograms(){}
    QC_DEBUGPROGRAM->addMethod("getAllPrograms", (q_method_n_t)DebugProgram_getAllPrograms, Public, QCF_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

    // int DebugProgram::getInterruptedCount(){}
    QC_DEBUGPROGRAM->addMethod("getInterruptedCount", (q_method_n_t)DebugProgram_getInterruptedCount, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing DebugProgram::onAttach(ProgramControl pgm, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onAttach", Public, QCF_NO_FLAGS, nothingTypeInfo, 3, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onDetach(ProgramControl pgm, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onDetach", Public, QCF_NO_FLAGS, nothingTypeInfo, 3, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onException(ProgramControl pgm, int statement, hash ex, reference dismiss, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onException", Public, QCF_NO_FLAGS, nothingTypeInfo, 6, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "statement", hashTypeInfo, QORE_PARAM_NO_ARG, "ex", referenceTypeInfo, QORE_PARAM_NO_ARG, "dismiss", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onExit(ProgramControl pgm, int statement, reference returnValue, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onExit", Public, QCF_NO_FLAGS, nothingTypeInfo, 5, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "statement", referenceTypeInfo, QORE_PARAM_NO_ARG, "returnValue", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onFunctionEnter(ProgramControl pgm, int statement, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onFunctionEnter", Public, QCF_NO_FLAGS, nothingTypeInfo, 4, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "statement", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onFunctionExit(ProgramControl pgm, int statement, reference returnValue, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onFunctionExit", Public, QCF_NO_FLAGS, nothingTypeInfo, 5, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "statement", referenceTypeInfo, QORE_PARAM_NO_ARG, "returnValue", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::onStep(ProgramControl pgm, int blockStatement, *int statement, *int breakpointId, reference flow, reference rs, reference rtsid);
    QC_DEBUGPROGRAM->addAbstractMethod("onStep", Public, QCF_NO_FLAGS, nothingTypeInfo, 7, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm", bigIntTypeInfo, QORE_PARAM_NO_ARG, "blockStatement", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "statement", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "breakpointId", referenceTypeInfo, QORE_PARAM_NO_ARG, "flow", referenceTypeInfo, QORE_PARAM_NO_ARG, "rs", referenceTypeInfo, QORE_PARAM_NO_ARG, "rtsid");

    // nothing DebugProgram::removeProgram(ProgramControl pgm){}
    QC_DEBUGPROGRAM->addMethod("removeProgram", (q_method_n_t)DebugProgram_removeProgram_C14ProgramControl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm");

    // nothing DebugProgram::waitForTerminationAndClear(){}
    QC_DEBUGPROGRAM->addMethod("waitForTerminationAndClear", (q_method_n_t)DebugProgram_waitForTerminationAndClear, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);
# 40 "QC_DebugProgram.qpp"
# 50 "QC_DebugProgram.qpp"
# 53 "QC_DebugProgram.qpp"
# 56 "QC_DebugProgram.qpp"
# 59 "QC_DebugProgram.qpp"
//  const DebugStopped = DBG_RS_STOPPED = 5,    // not needed
    ns.addConstant("DebugDetach", ((int64)DBG_RS_DETACH));
    ns.addConstant("DebugRun", ((int64)DBG_RS_RUN));
    ns.addConstant("DebugStep", ((int64)DBG_RS_STEP));
    ns.addConstant("DebugStepOver", ((int64)DBG_RS_STEP_OVER));
    ns.addConstant("DebugUntilReturn", ((int64)DBG_RS_UNTIL_RETURN));
    ns.addConstant("DebugFlowBreak", ((int64)RC_BREAK));
    ns.addConstant("DebugFlowContinue", ((int64)RC_CONTINUE));
    ns.addConstant("DebugFlowReturn", ((int64)RC_RETURN));

    return QC_DEBUGPROGRAM;
}
