/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListReverseIterator.qpp ListReverseIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreListIterator.h"

extern QoreClass* QC_LISTITERATOR;

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreListReverseIterator : public QoreListIterator {
public:
   DLLLOCAL QoreListReverseIterator(const QoreListNode* l) : QoreListIterator(l) {
   }

   DLLLOCAL QoreListReverseIterator(const QoreListReverseIterator& old) : QoreListIterator(old) {
   }

   DLLLOCAL virtual const char* getName() const {
      return "ListReverseIterator";
   }
};
/* Qore class Qore::ListReverseIterator */

qore_classid_t CID_LISTREVERSEITERATOR;
QoreClass* QC_LISTREVERSEITERATOR;

// ListReverseIterator::constructor(softlist<auto> l) {}
static void ListReverseIterator_constructor_C14softlist_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 93 "QC_ListReverseIterator.qpp"
self->setPrivate(CID_LISTREVERSEITERATOR, new QoreListReverseIterator(l));
}

// ListReverseIterator::copy() {}
static void ListReverseIterator_copy(QoreObject* self, QoreObject* old, QoreListReverseIterator* i, ExceptionSink* xsink) {
# 103 "QC_ListReverseIterator.qpp"
self->setPrivate(CID_LISTREVERSEITERATOR, new QoreListReverseIterator(*i));
}

// bool ListReverseIterator::first(){}
static QoreValue ListReverseIterator_first(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 166 "QC_ListReverseIterator.qpp"
    return i->last();
}

// bool ListReverseIterator::last(){}
static QoreValue ListReverseIterator_last(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 181 "QC_ListReverseIterator.qpp"
    return i->first();
}

// bool ListReverseIterator::next(){}
static QoreValue ListReverseIterator_next(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 125 "QC_ListReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->prev();
}

// bool ListReverseIterator::prev(){}
static QoreValue ListReverseIterator_prev(QoreObject* self, QoreListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 149 "QC_ListReverseIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->next();
}

DLLLOCAL void preinitListReverseIteratorClass() {
    QC_LISTREVERSEITERATOR = new QoreClass("ListReverseIterator", "::Qore::ListReverseIterator", QDOM_DEFAULT);
    CID_LISTREVERSEITERATOR = QC_LISTREVERSEITERATOR->getID();
    QC_LISTREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initListReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_LISTREVERSEITERATOR)
        preinitListReverseIteratorClass();

    // set parent class
    assert(QC_LISTITERATOR);
    QC_LISTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_LISTITERATOR);

    // ListReverseIterator::constructor(softlist<auto> l) {}
    QC_LISTREVERSEITERATOR->addConstructor(ListReverseIterator_constructor_C14softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "l");

    // ListReverseIterator::copy() {}
    QC_LISTREVERSEITERATOR->setCopy((q_copy_t)ListReverseIterator_copy);

    // bool ListReverseIterator::first(){}
    QC_LISTREVERSEITERATOR->addMethod("first", (q_method_n_t)ListReverseIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ListReverseIterator::last(){}
    QC_LISTREVERSEITERATOR->addMethod("last", (q_method_n_t)ListReverseIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ListReverseIterator::next(){}
    QC_LISTREVERSEITERATOR->addMethod("next", (q_method_n_t)ListReverseIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool ListReverseIterator::prev(){}
    QC_LISTREVERSEITERATOR->addMethod("prev", (q_method_n_t)ListReverseIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    return QC_LISTREVERSEITERATOR;
}
