/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StreamWriter.qpp StreamWriter class definition */
/*
  Qore Programming Language

  Copyright (C) 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/OutputStream.h"
#include "qore/intern/StreamWriter.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"

DLLEXPORT extern QoreClass* QC_OUTPUTSTREAM;
/* Qore class Qore::StreamWriter */

qore_classid_t CID_STREAMWRITER;
QoreClass* QC_STREAMWRITER;

// StreamWriter::constructor(Qore::OutputStream os, *string encoding) {}
static void StreamWriter_constructor_C12OutputStreamNs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "StreamWriter::constructor()", "Qore::OutputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 62 "QC_StreamWriter.qpp"
SimpleRefHolder<StreamWriter> writer(new StreamWriter(xsink, os, encoding ? QEM.findCreate(encoding) : QCS_DEFAULT));
   if (*xsink)
      return;
   self->setPrivate(CID_STREAMWRITER, writer.release());
   qore_object_private* o = qore_object_private::get(*self);
   const qore_class_private* cls = qore_class_private::get(*QC_STREAMWRITER);
   o->setValueIntern(cls, "os", static_cast<QoreObject *>(obj_os->refSelf()), xsink);
}

// nothing StreamWriter::f_printf(string fmt, ...){}
static QoreValue StreamWriter_f_printf_VsVV(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 159 "QC_StreamWriter.qpp"
    sw->f_printf(args, xsink);
    return QoreValue();
}

// nothing StreamWriter::f_vprintf(string fmt, any fmt_args){}
static QoreValue StreamWriter_f_vprintf_VsVa(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 178 "QC_StreamWriter.qpp"
    sw->f_vprintf(args, xsink);
    return QoreValue();
}

// string StreamWriter::getEncoding(){}
static QoreValue StreamWriter_getEncoding(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 77 "QC_StreamWriter.qpp"
    return new QoreStringNode(sw->getEncoding()->getCode());
}

// OutputStream StreamWriter::getOutputStream(){}
static QoreValue StreamWriter_getOutputStream(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 88 "QC_StreamWriter.qpp"
    return self->getReferencedMemberNoMethod("os", xsink);
}

// nothing StreamWriter::print(string str){}
static QoreValue StreamWriter_print_Vs(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 104 "QC_StreamWriter.qpp"
    sw->print(str, xsink);
    return QoreValue();
}

// nothing StreamWriter::printf(string fmt, ...){}
static QoreValue StreamWriter_printf_VsVV(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "QC_StreamWriter.qpp"
    sw->printf(args, xsink);
    return QoreValue();
}

// nothing StreamWriter::vprintf(string fmt, any fmt_args){}
static QoreValue StreamWriter_vprintf_VsVa(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 141 "QC_StreamWriter.qpp"
    sw->vprintf(args, xsink);
    return QoreValue();
}

// nothing StreamWriter::write(string str){}
static QoreValue StreamWriter_write_Vs(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 192 "QC_StreamWriter.qpp"
    sw->print(str, xsink);
    return QoreValue();
}

// nothing StreamWriter::write(binary b){}
static QoreValue StreamWriter_write_Vy(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
# 204 "QC_StreamWriter.qpp"
    sw->write(b, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei1(int c){}
static QoreValue StreamWriter_writei1_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 c = HARD_QORE_VALUE_INT(args, 0);
# 216 "QC_StreamWriter.qpp"
    sw->writei1(c, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei2(int s){}
static QoreValue StreamWriter_writei2_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 s = HARD_QORE_VALUE_INT(args, 0);
# 228 "QC_StreamWriter.qpp"
    sw->writei2(s, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei2LSB(int s){}
static QoreValue StreamWriter_writei2LSB_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 s = HARD_QORE_VALUE_INT(args, 0);
# 264 "QC_StreamWriter.qpp"
    sw->writei2LSB(s, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei4(int i){}
static QoreValue StreamWriter_writei4_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 240 "QC_StreamWriter.qpp"
    sw->writei4(i, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei4LSB(int i){}
static QoreValue StreamWriter_writei4LSB_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 276 "QC_StreamWriter.qpp"
    sw->writei4LSB(i, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei8(int i){}
static QoreValue StreamWriter_writei8_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 252 "QC_StreamWriter.qpp"
    sw->writei8(i, xsink);
    return QoreValue();
}

// nothing StreamWriter::writei8LSB(int i){}
static QoreValue StreamWriter_writei8LSB_Vi(QoreObject* self, StreamWriter* sw, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 288 "QC_StreamWriter.qpp"
    sw->writei8LSB(i, xsink);
    return QoreValue();
}

DLLLOCAL void preinitStreamWriterClass() {
    QC_STREAMWRITER = new QoreClass("StreamWriter", "::Qore::StreamWriter", QDOM_DEFAULT);
    CID_STREAMWRITER = QC_STREAMWRITER->getID();
    QC_STREAMWRITER->setSystem();
}

DLLLOCAL QoreClass* initStreamWriterClass(QoreNamespace& ns) {
    if (!QC_STREAMWRITER)
        preinitStreamWriterClass();

    // private:internal members
    QC_STREAMWRITER->addMember("os", Internal, QC_OUTPUTSTREAM->getTypeInfo());

    // StreamWriter::constructor(Qore::OutputStream os, *string encoding) {}
    QC_STREAMWRITER->addConstructor(StreamWriter_constructor_C12OutputStreamNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 2, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing StreamWriter::f_printf(string fmt, ...){}
    QC_STREAMWRITER->addMethod("f_printf", (q_method_n_t)StreamWriter_f_printf_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // nothing StreamWriter::f_vprintf(string fmt, any fmt_args){}
    QC_STREAMWRITER->addMethod("f_vprintf", (q_method_n_t)StreamWriter_f_vprintf_VsVa, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", anyTypeInfo, QORE_PARAM_NO_ARG, "fmt_args");

    // string StreamWriter::getEncoding(){}
    QC_STREAMWRITER->addMethod("getEncoding", (q_method_n_t)StreamWriter_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // OutputStream StreamWriter::getOutputStream(){}
    QC_STREAMWRITER->addMethod("getOutputStream", (q_method_n_t)StreamWriter_getOutputStream, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_OUTPUTSTREAM->getTypeInfo());

    // nothing StreamWriter::print(string str){}
    QC_STREAMWRITER->addMethod("print", (q_method_n_t)StreamWriter_print_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");

    // nothing StreamWriter::printf(string fmt, ...){}
    QC_STREAMWRITER->addMethod("printf", (q_method_n_t)StreamWriter_printf_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // nothing StreamWriter::vprintf(string fmt, any fmt_args){}
    QC_STREAMWRITER->addMethod("vprintf", (q_method_n_t)StreamWriter_vprintf_VsVa, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt", anyTypeInfo, QORE_PARAM_NO_ARG, "fmt_args");

    // nothing StreamWriter::write(string str){}
    QC_STREAMWRITER->addMethod("write", (q_method_n_t)StreamWriter_write_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "str");

    // nothing StreamWriter::write(binary b){}
    QC_STREAMWRITER->addMethod("write", (q_method_n_t)StreamWriter_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "b");

    // nothing StreamWriter::writei1(int c){}
    QC_STREAMWRITER->addMethod("writei1", (q_method_n_t)StreamWriter_writei1_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "c");

    // nothing StreamWriter::writei2(int s){}
    QC_STREAMWRITER->addMethod("writei2", (q_method_n_t)StreamWriter_writei2_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "s");

    // nothing StreamWriter::writei2LSB(int s){}
    QC_STREAMWRITER->addMethod("writei2LSB", (q_method_n_t)StreamWriter_writei2LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "s");

    // nothing StreamWriter::writei4(int i){}
    QC_STREAMWRITER->addMethod("writei4", (q_method_n_t)StreamWriter_writei4_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // nothing StreamWriter::writei4LSB(int i){}
    QC_STREAMWRITER->addMethod("writei4LSB", (q_method_n_t)StreamWriter_writei4LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // nothing StreamWriter::writei8(int i){}
    QC_STREAMWRITER->addMethod("writei8", (q_method_n_t)StreamWriter_writei8_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    // nothing StreamWriter::writei8LSB(int i){}
    QC_STREAMWRITER->addMethod("writei8LSB", (q_method_n_t)StreamWriter_writei8LSB_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "i");

    return QC_STREAMWRITER;
}
