/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_TreeMap.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_TreeMap.h"
/* Qore class Qore::TreeMap */

qore_classid_t CID_TREEMAP;
QoreClass* QC_TREEMAP;

// TreeMap::constructor() {}
static void TreeMap_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 62 "QC_TreeMap.qpp"
self->setPrivate(CID_TREEMAP, new TreeMapData);
}

// TreeMap::copy() {}
static void TreeMap_copy(QoreObject* self, QoreObject* old, TreeMapData* tm, ExceptionSink* xsink) {
# 77 "QC_TreeMap.qpp"
xsink->raiseException("TREEMAP-COPY-ERROR", "objects of this class cannot be copied");
}

// TreeMap::destructor() {}
static void TreeMap_destructor(QoreObject* self, TreeMapData* tm, ExceptionSink* xsink) {
# 69 "QC_TreeMap.qpp"
tm->deref(xsink);
}

// auto TreeMap::get(string path, *reference<*string> unmatched){}
static QoreValue TreeMap_get_VsC19_reference__string_(QoreObject* self, TreeMapData* tm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* unmatched = get_param_value(args, 1).get<const ReferenceNode>();
# 120 "QC_TreeMap.qpp"
    return tm->get(path, unmatched, xsink);
}

// *hash TreeMap::getAll(){}
static QoreValue TreeMap_getAll(QoreObject* self, TreeMapData* tm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 140 "QC_TreeMap.qpp"
    return tm->getAll();
}

// nothing TreeMap::put(string path, auto value){}
static QoreValue TreeMap_put_VsVA(QoreObject* self, TreeMapData* tm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 88 "QC_TreeMap.qpp"
    tm->put(path, value, xsink);
    return QoreValue();
}

// auto TreeMap::take(string path){}
static QoreValue TreeMap_take_Vs(QoreObject* self, TreeMapData* tm, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 133 "QC_TreeMap.qpp"
    return tm->take(path, xsink);
}

DLLLOCAL void preinitTreeMapClass() {
    QC_TREEMAP = new QoreClass("TreeMap", "::Qore::TreeMap", QDOM_DEFAULT);
    CID_TREEMAP = QC_TREEMAP->getID();
    QC_TREEMAP->setSystem();
}

DLLLOCAL QoreClass* initTreeMapClass(QoreNamespace& ns) {
    if (!QC_TREEMAP)
        preinitTreeMapClass();

    // TreeMap::constructor() {}
    QC_TREEMAP->addConstructor(TreeMap_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // TreeMap::copy() {}
    QC_TREEMAP->setCopy((q_copy_t)TreeMap_copy);

    // TreeMap::destructor() {}
    QC_TREEMAP->setDestructor((q_destructor_t)TreeMap_destructor);

    // auto TreeMap::get(string path, *reference<*string> unmatched){}
    QC_TREEMAP->addMethod("get", (q_method_n_t)TreeMap_get_VsC19_reference__string_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "path", qore_get_complex_reference_or_nothing_type(stringOrNothingTypeInfo), QORE_PARAM_NO_ARG, "unmatched");

    // *hash TreeMap::getAll(){}
    QC_TREEMAP->addMethod("getAll", (q_method_n_t)TreeMap_getAll, Public, QCF_CONSTANT, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // nothing TreeMap::put(string path, auto value){}
    QC_TREEMAP->addMethod("put", (q_method_n_t)TreeMap_put_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "path", autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // auto TreeMap::take(string path){}
    QC_TREEMAP->addMethod("take", (q_method_n_t)TreeMap_take_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    return QC_TREEMAP;
}
