/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    qc_qore.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_File.h"

DLLEXPORT extern QoreClass* QC_FILEINPUTSTREAM;
DLLEXPORT extern QoreClass* QC_FILEOUTPUTSTREAM;

#ifdef WORDS_BIGENDIAN
#define Q_MACHINE_MSB 1
#else
#define Q_MACHINE_MSB 0
#endif

#ifdef _Q_WINDOWS
#define Q_IS_WINDOWS 1
#define Q_IS_UNIX    0
#else
#define Q_IS_WINDOWS 0
#define Q_IS_UNIX    1
#endif

DLLLOCAL void init_qore_constants(QoreNamespace& ns) {
# 53 "qc_qore.qpp"
# 62 "qc_qore.qpp"
# 65 "qc_qore.qpp"
# 68 "qc_qore.qpp"
# 71 "qc_qore.qpp"
# 74 "qc_qore.qpp"
# 77 "qc_qore.qpp"
# 80 "qc_qore.qpp"
# 83 "qc_qore.qpp"
# 86 "qc_qore.qpp"
# 89 "qc_qore.qpp"
# 92 "qc_qore.qpp"
# 95 "qc_qore.qpp"
# 98 "qc_qore.qpp"
    ns.addConstant("NT_ALL", ((int64)NT_ALL));
    ns.addConstant("NT_BINARY", ((int64)NT_BINARY));
    ns.addConstant("NT_BOOLEAN", ((int64)NT_BOOLEAN));
    ns.addConstant("NT_CALLREF", ((int64)NT_FUNCREF));
    ns.addConstant("NT_CLOSURE", ((int64)NT_RUNTIME_CLOSURE));
    ns.addConstant("NT_DATE", ((int64)NT_DATE));
    ns.addConstant("NT_FLOAT", ((int64)NT_FLOAT));
    ns.addConstant("NT_HASH", ((int64)NT_HASH));
    ns.addConstant("NT_INT", ((int64)NT_INT));
    ns.addConstant("NT_LIST", ((int64)NT_LIST));
    ns.addConstant("NT_NOTHING", ((int64)NT_NOTHING));
    ns.addConstant("NT_NULL", ((int64)NT_NULL));
    ns.addConstant("NT_NUMBER", ((int64)NT_NUMBER));
    ns.addConstant("NT_OBJECT", ((int64)NT_OBJECT));
    ns.addConstant("NT_STRING", ((int64)NT_STRING));
# 105 "qc_qore.qpp"
# 128 "qc_qore.qpp"
    QoreHashNode* hash_TypeCodeMap = new QoreHashNode(autoTypeInfo);
    hash_TypeCodeMap->setKeyValue("0", new QoreStringNode("nothing"), 0);
    hash_TypeCodeMap->setKeyValue("1", new QoreStringNode("integer"), 0);
    hash_TypeCodeMap->setKeyValue("2", new QoreStringNode("float"), 0);
    hash_TypeCodeMap->setKeyValue("3", new QoreStringNode("string"), 0);
    hash_TypeCodeMap->setKeyValue("4", new QoreStringNode("date"), 0);
    hash_TypeCodeMap->setKeyValue("5", new QoreStringNode("bool"), 0);
    hash_TypeCodeMap->setKeyValue("6", new QoreStringNode("NULL"), 0);
    hash_TypeCodeMap->setKeyValue("7", new QoreStringNode("binary"), 0);
    hash_TypeCodeMap->setKeyValue("8", new QoreStringNode("list"), 0);
    hash_TypeCodeMap->setKeyValue("9", new QoreStringNode("hash"), 0);
    hash_TypeCodeMap->setKeyValue("10", new QoreStringNode("object"), 0);
    hash_TypeCodeMap->setKeyValue("11", new QoreStringNode("number"), 0);
    hash_TypeCodeMap->setKeyValue("29", new QoreStringNode("call reference"), 0);
    hash_TypeCodeMap->setKeyValue("32", new QoreStringNode("closure"), 0);

    ns.addConstant("TypeCodeMap", hash_TypeCodeMap);
    QoreHashNode* hash_TypeNameMap = new QoreHashNode(autoTypeInfo);
    hash_TypeNameMap->setKeyValue("nothing", ((int64)NT_NOTHING), 0);
    hash_TypeNameMap->setKeyValue("integer", ((int64)NT_INT), 0);
    hash_TypeNameMap->setKeyValue("float", ((int64)NT_FLOAT), 0);
    hash_TypeNameMap->setKeyValue("number", ((int64)NT_NUMBER), 0);
    hash_TypeNameMap->setKeyValue("string", ((int64)NT_STRING), 0);
    hash_TypeNameMap->setKeyValue("date", ((int64)NT_DATE), 0);
    hash_TypeNameMap->setKeyValue("bool", ((int64)NT_BOOLEAN), 0);
    hash_TypeNameMap->setKeyValue("NULL", ((int64)NT_NULL), 0);
    hash_TypeNameMap->setKeyValue("binary", ((int64)NT_BINARY), 0);
    hash_TypeNameMap->setKeyValue("list", ((int64)NT_LIST), 0);
    hash_TypeNameMap->setKeyValue("hash", ((int64)NT_HASH), 0);
    hash_TypeNameMap->setKeyValue("object", ((int64)NT_OBJECT), 0);
    hash_TypeNameMap->setKeyValue("call reference", ((int64)NT_FUNCREF), 0);
    hash_TypeNameMap->setKeyValue("closure", ((int64)NT_RUNTIME_CLOSURE), 0);

    ns.addConstant("TypeNameMap", hash_TypeNameMap);
# 147 "qc_qore.qpp"
    ns.addConstant("False", ((bool)false));
    ns.addConstant("True", ((bool)true));
# 157 "qc_qore.qpp"
    ns.addConstant("NOTHING", (&Nothing));
    ns.addConstant("NULL", (&Null));
# 167 "qc_qore.qpp"
    ns.addConstant("ET_System", new QoreStringNode("System"));
    ns.addConstant("ET_User", new QoreStringNode("User"));
# 177 "qc_qore.qpp"
# 183 "qc_qore.qpp"
# 186 "qc_qore.qpp"
    ns.addConstant("CT_Builtin", ((int64)CT_BUILTIN));
    ns.addConstant("CT_NewThread", ((int64)CT_NEWTHREAD));
    ns.addConstant("CT_Rethrow", ((int64)CT_RETHROW));
    ns.addConstant("CT_User", ((int64)CT_USER));
# 193 "qc_qore.qpp"
# 199 "qc_qore.qpp"
# 202 "qc_qore.qpp"
# 205 "qc_qore.qpp"
# 208 "qc_qore.qpp"
# 211 "qc_qore.qpp"
# 214 "qc_qore.qpp"
# 217 "qc_qore.qpp"
# 220 "qc_qore.qpp"
# 223 "qc_qore.qpp"
# 226 "qc_qore.qpp"
# 229 "qc_qore.qpp"
# 232 "qc_qore.qpp"
# 235 "qc_qore.qpp"
    ns.addConstant("Build", (qore_build_number));
    ns.addConstant("BuildHost", (new QoreStringNode(qore_build_host)));
    ns.addConstant("CFLAGS", (new QoreStringNode(qore_cflags)));
    ns.addConstant("Compiler", (new QoreStringNode(qore_cplusplus_compiler)));
    ns.addConstant("DirSep", (new QoreStringNode(QORE_DIR_SEP_STR)));
    ns.addConstant("LDFLAGS", (new QoreStringNode(qore_ldflags)));
    ns.addConstant("MACHINE_MSB", ((bool)Q_MACHINE_MSB));
    ns.addConstant("PathSep", (new QoreStringNode(QORE_PATH_SEP_STR)));
    QoreHashNode* hash_Platform = new QoreHashNode(autoTypeInfo);
    hash_Platform->setKeyValue("OS", (new QoreStringNode(TARGET_OS)), 0);
    hash_Platform->setKeyValue("CPU", (new QoreStringNode(TARGET_ARCH)), 0);
    hash_Platform->setKeyValue("Unix", ((bool)Q_IS_UNIX), 0);
    hash_Platform->setKeyValue("Windows", ((bool)Q_IS_WINDOWS), 0);

    ns.addConstant("Platform", hash_Platform);
    ns.addConstant("PlatformCPU", (new QoreStringNode(TARGET_ARCH)));
    ns.addConstant("PlatformOS", (new QoreStringNode(TARGET_OS)));
    ns.addConstant("VersionMajor", (qore_version_major));
    ns.addConstant("VersionMinor", (qore_version_minor));
    ns.addConstant("VersionString", (new QoreStringNode(qore_version_string)));
    ns.addConstant("VersionSub", (qore_version_sub));
# 252 "qc_qore.qpp"
# 258 "qc_qore.qpp"
# 261 "qc_qore.qpp"
    ns.addConstant("SOURCE_FILE", ((int64)QORE_SOURCE_FILE));
    ns.addConstant("SOURCE_FTPCLIENT", ((int64)QORE_SOURCE_FTPCLIENT));
    ns.addConstant("SOURCE_HTTPCLIENT", ((int64)QORE_SOURCE_HTTPCLIENT));
    ns.addConstant("SOURCE_SOCKET", ((int64)QORE_SOURCE_SOCKET));
# 268 "qc_qore.qpp"
# 279 "qc_qore.qpp"
    QoreHashNode* hash_EVENT_MAP = new QoreHashNode(autoTypeInfo);
    hash_EVENT_MAP->setKeyValue("1", new QoreStringNode("PACKET_READ"), 0);
    hash_EVENT_MAP->setKeyValue("2", new QoreStringNode("PACKET_SENT"), 0);
    hash_EVENT_MAP->setKeyValue("3", new QoreStringNode("HTTP_CONTENT_LENGTH"), 0);
    hash_EVENT_MAP->setKeyValue("4", new QoreStringNode("HTTP_CHUNKED_START"), 0);
    hash_EVENT_MAP->setKeyValue("5", new QoreStringNode("HTTP_CHUNKED_END"), 0);
    hash_EVENT_MAP->setKeyValue("6", new QoreStringNode("HTTP_REDIRECT"), 0);
    hash_EVENT_MAP->setKeyValue("7", new QoreStringNode("CHANNEL_CLOSED"), 0);
    hash_EVENT_MAP->setKeyValue("8", new QoreStringNode("DELETED"), 0);
    hash_EVENT_MAP->setKeyValue("9", new QoreStringNode("FTP_SEND_MESSAGE"), 0);
    hash_EVENT_MAP->setKeyValue("10", new QoreStringNode("FTP_MESSAGE_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("11", new QoreStringNode("HOSTNAME_LOOKUP"), 0);
    hash_EVENT_MAP->setKeyValue("12", new QoreStringNode("HOSTNAME_RESOLVED"), 0);
    hash_EVENT_MAP->setKeyValue("13", new QoreStringNode("HTTP_SEND_MESSAGE"), 0);
    hash_EVENT_MAP->setKeyValue("14", new QoreStringNode("HTTP_MESSAGE_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("15", new QoreStringNode("HTTP_FOOTERS_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("16", new QoreStringNode("HTTP_CHUNKED_DATA_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("17", new QoreStringNode("HTTP_CHUNK_SIZE"), 0);
    hash_EVENT_MAP->setKeyValue("18", new QoreStringNode("CONNECTING"), 0);
    hash_EVENT_MAP->setKeyValue("19", new QoreStringNode("CONNECTED"), 0);
    hash_EVENT_MAP->setKeyValue("20", new QoreStringNode("START_SSL"), 0);
    hash_EVENT_MAP->setKeyValue("21", new QoreStringNode("SSL_ESTABLISHED"), 0);
    hash_EVENT_MAP->setKeyValue("22", new QoreStringNode("OPEN_FILE"), 0);
    hash_EVENT_MAP->setKeyValue("23", new QoreStringNode("FILE_OPENED"), 0);
    hash_EVENT_MAP->setKeyValue("24", new QoreStringNode("DATA_READ"), 0);
    hash_EVENT_MAP->setKeyValue("25", new QoreStringNode("DATA_WRITTEN"), 0);
    hash_EVENT_MAP->setKeyValue("26", new QoreStringNode("HTTP_CHUNKED_DATA_READ"), 0);
    hash_EVENT_MAP->setKeyValue("27", new QoreStringNode("HTTP_CHUNKED_DATA_SENT"), 0);
    hash_EVENT_MAP->setKeyValue("28", new QoreStringNode("HTTP_FOOTERS_SENT"), 0);
    hash_EVENT_MAP->setKeyValue("29", new QoreStringNode("SOCKET_DATA_READ"), 0);
    hash_EVENT_MAP->setKeyValue("30", new QoreStringNode("SOCKET_DATA_SENT"), 0);
    hash_EVENT_MAP->setKeyValue("31", new QoreStringNode("HTTP_HEADERS_READ"), 0);

    ns.addConstant("EVENT_MAP", hash_EVENT_MAP);
    QoreHashNode* hash_EVENT_SOURCE_MAP = new QoreHashNode(autoTypeInfo);
    hash_EVENT_SOURCE_MAP->setKeyValue("1", new QoreStringNode("SOCKET"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("2", new QoreStringNode("HTTPCLIENT"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("3", new QoreStringNode("FTPCLIENT"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("4", new QoreStringNode("FILE"), 0);

    ns.addConstant("EVENT_SOURCE_MAP", hash_EVENT_SOURCE_MAP);
# 315 "qc_qore.qpp"
# 321 "qc_qore.qpp"
# 324 "qc_qore.qpp"
# 327 "qc_qore.qpp"
# 330 "qc_qore.qpp"
# 333 "qc_qore.qpp"
# 336 "qc_qore.qpp"
# 339 "qc_qore.qpp"
# 342 "qc_qore.qpp"
# 345 "qc_qore.qpp"
# 348 "qc_qore.qpp"
# 351 "qc_qore.qpp"
# 354 "qc_qore.qpp"
# 357 "qc_qore.qpp"
# 360 "qc_qore.qpp"
# 363 "qc_qore.qpp"
# 366 "qc_qore.qpp"
# 369 "qc_qore.qpp"
# 372 "qc_qore.qpp"
# 375 "qc_qore.qpp"
# 378 "qc_qore.qpp"
# 381 "qc_qore.qpp"
# 384 "qc_qore.qpp"
# 387 "qc_qore.qpp"
# 390 "qc_qore.qpp"
# 393 "qc_qore.qpp"
# 398 "qc_qore.qpp"
# 403 "qc_qore.qpp"
# 408 "qc_qore.qpp"
# 413 "qc_qore.qpp"
    ns.addConstant("EVENT_CHANNEL_CLOSED", ((int64)QORE_EVENT_CHANNEL_CLOSED));
    ns.addConstant("EVENT_CONNECTED", ((int64)QORE_EVENT_CONNECTED));
    ns.addConstant("EVENT_CONNECTING", ((int64)QORE_EVENT_CONNECTING));
    ns.addConstant("EVENT_DATA_READ", ((int64)QORE_EVENT_DATA_READ));
    ns.addConstant("EVENT_DATA_WRITTEN", ((int64)QORE_EVENT_DATA_WRITTEN));
    ns.addConstant("EVENT_DELETED", ((int64)QORE_EVENT_DELETED));
    ns.addConstant("EVENT_FILE_OPENED", ((int64)QORE_EVENT_FILE_OPENED));
    ns.addConstant("EVENT_FTP_MESSAGE_RECEIVED", ((int64)QORE_EVENT_FTP_MESSAGE_RECEIVED));
    ns.addConstant("EVENT_FTP_SEND_MESSAGE", ((int64)QORE_EVENT_FTP_SEND_MESSAGE));
    ns.addConstant("EVENT_HOSTNAME_LOOKUP", ((int64)QORE_EVENT_HOSTNAME_LOOKUP));
    ns.addConstant("EVENT_HOSTNAME_RESOLVED", ((int64)QORE_EVENT_HOSTNAME_RESOLVED));
    ns.addConstant("EVENT_HTTP_CHUNKED_DATA_READ", ((int64)QORE_EVENT_HTTP_CHUNKED_DATA_READ));
    ns.addConstant("EVENT_HTTP_CHUNKED_DATA_RECEIVED", ((int64)QORE_EVENT_HTTP_CHUNKED_DATA_RECEIVED));
    ns.addConstant("EVENT_HTTP_CHUNKED_DATA_SENT", ((int64)QORE_EVENT_HTTP_CHUNKED_DATA_SENT));
    ns.addConstant("EVENT_HTTP_CHUNKED_END", ((int64)QORE_EVENT_HTTP_CHUNKED_END));
    ns.addConstant("EVENT_HTTP_CHUNKED_FOOTERS_SENT", ((int64)QORE_EVENT_HTTP_FOOTERS_SENT));
    ns.addConstant("EVENT_HTTP_CHUNKED_START", ((int64)QORE_EVENT_HTTP_CHUNKED_START));
    ns.addConstant("EVENT_HTTP_CHUNK_SIZE", ((int64)QORE_EVENT_HTTP_CHUNK_SIZE));
    ns.addConstant("EVENT_HTTP_CONTENT_LENGTH", ((int64)QORE_EVENT_HTTP_CONTENT_LENGTH));
    ns.addConstant("EVENT_HTTP_FOOTERS_RECEIVED", ((int64)QORE_EVENT_HTTP_FOOTERS_RECEIVED));
    ns.addConstant("EVENT_HTTP_HEADERS_READ", ((int64)QORE_EVENT_HTTP_HEADERS_READ));
    ns.addConstant("EVENT_HTTP_MESSAGE_RECEIVED", ((int64)QORE_EVENT_HTTP_MESSAGE_RECEIVED));
    ns.addConstant("EVENT_HTTP_REDIRECT", ((int64)QORE_EVENT_HTTP_REDIRECT));
    ns.addConstant("EVENT_HTTP_SEND_MESSAGE", ((int64)QORE_EVENT_HTTP_SEND_MESSAGE));
    ns.addConstant("EVENT_OPEN_FILE", ((int64)QORE_EVENT_OPEN_FILE));
    ns.addConstant("EVENT_PACKET_READ", ((int64)QORE_EVENT_PACKET_READ));
    ns.addConstant("EVENT_PACKET_SENT", ((int64)QORE_EVENT_PACKET_SENT));
    ns.addConstant("EVENT_SOCKET_DATA_READ", ((int64)QORE_EVENT_SOCKET_DATA_READ));
    ns.addConstant("EVENT_SOCKET_DATA_SENT", ((int64)QORE_EVENT_SOCKET_DATA_SENT));
    ns.addConstant("EVENT_SSL_ESTABLISHED", ((int64)QORE_EVENT_SSL_ESTABLISHED));
    ns.addConstant("EVENT_START_SSL", ((int64)QORE_EVENT_START_SSL));
# 424 "qc_qore.qpp"
# 433 "qc_qore.qpp"
# 436 "qc_qore.qpp"
# 439 "qc_qore.qpp"
# 442 "qc_qore.qpp"
    ns.addConstant("stderr", (QC_FILE->execSystemConstructor(2)));
    ns.addConstant("stderr_stream", (QC_FILEOUTPUTSTREAM->execSystemConstructor(2)));
    ns.addConstant("stdin", (QC_FILE->execSystemConstructor(0)));
    ns.addConstant("stdin_stream", (QC_FILEINPUTSTREAM->execSystemConstructor(0)));
    ns.addConstant("stdout", (QC_FILE->execSystemConstructor(1)));
    ns.addConstant("stdout_stream", (QC_FILEOUTPUTSTREAM->execSystemConstructor(1)));

}
