/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_DestructorMethod.cpp DestructorMethod class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include "QC_DestructorMethod.h"
#include "QC_Class.h"
/* Qore class Qore::Reflection::DestructorMethod */

qore_classid_t CID_DESTRUCTORMETHOD;
QoreClass* QC_DESTRUCTORMETHOD;

// auto DestructorMethod::call(object obj){}
static QoreValue DestructorMethod_call_Vo(QoreObject* self, QoreReflectionMethod* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 112 "QC_DestructorMethod.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // we have to explictly test object compatibility here
    if (check_call(obj, *m->m, xsink)) {
        return QoreValue();
    }

    obj->doDelete(xsink);
    return QoreValue();
}

// DestructorMethod::constructor(Class cls) {}
static void DestructorMethod_constructor_C5Class(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cls, QoreReflectionClass, args, 0, CID_CLASS, "DestructorMethod::constructor()", "Class", xsink);
    if (*xsink)
        return;
# 54 "QC_DestructorMethod.qpp"
ReferenceHolder<QoreReflectionClass> holder(cls, xsink);

    // get exclusive access to program object
    QoreExternalProgramContextHelper pch(xsink, cls->pgm);
    if (*xsink) {
        return;
    }

    // the class pointer must be valid here
    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(cls->cls, "destructor", xsink), xsink);
    if (*xsink) {
        return;
    }

    assert(m->mtype == MT_Destructor);

    self->setPrivate(CID_DESTRUCTORMETHOD, m.release());
}

// DestructorMethod::constructor(string class_name) {}
static void DestructorMethod_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
# 85 "QC_DestructorMethod.qpp"
TempEncodingHelper npath(class_name, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }

    ReferenceHolder<QoreReflectionMethod> m(new QoreReflectionMethod(npath->c_str(), "destructor", xsink), xsink);
    if (*xsink) {
        return;
    }

    assert(m->mtype == MT_Destructor);

    self->setPrivate(CID_DESTRUCTORMETHOD, m.release());
}

DLLLOCAL void preinitDestructorMethodClass() {
    QC_DESTRUCTORMETHOD = new QoreClass("DestructorMethod", "::Qore::Reflection::DestructorMethod", QDOM_REFLECTION);
    CID_DESTRUCTORMETHOD = QC_DESTRUCTORMETHOD->getID();
    QC_DESTRUCTORMETHOD->setSystem();
}

DLLLOCAL QoreClass* initDestructorMethodClass(QoreNamespace& ns) {
    if (!QC_DESTRUCTORMETHOD)
        preinitDestructorMethodClass();

    // set parent class
    assert(QC_ABSTRACTMETHOD);
    QC_DESTRUCTORMETHOD->addBuiltinVirtualBaseClass(QC_ABSTRACTMETHOD);

    // auto DestructorMethod::call(object obj){}
    QC_DESTRUCTORMETHOD->addMethod("call", (q_method_n_t)DestructorMethod_call_Vo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    // DestructorMethod::constructor(Class cls) {}
    QC_DESTRUCTORMETHOD->addConstructor(DestructorMethod_constructor_C5Class, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, QC_CLASS->getTypeInfo(), QORE_PARAM_NO_ARG, "cls");

    // DestructorMethod::constructor(string class_name) {}
    QC_DESTRUCTORMETHOD->addConstructor(DestructorMethod_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "class_name");

    return QC_DESTRUCTORMETHOD;
}
