/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_TypedHashMember.cpp TypedHashMember class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_Class.h>
#include <QC_TypedHashMember.h>

static int check_member(const QoreHashNode* h, const char* mname, const TypedHashDecl* th, ExceptionSink* xsink) {
    if (h->getHashDecl() != th) {
        xsink->raiseException("MEMBER-ERROR", "hash was not declared as hashdecl '%s' (referencing member '%s')", th->getName(), mname);
        return -1;
    }
    return 0;
}
/* Qore class Qore::Reflection::TypedHashMember */

qore_classid_t CID_TYPEDHASHMEMBER;
QoreClass* QC_TYPEDHASHMEMBER;

// TypedHashMember::constructor() {}
static void TypedHashMember_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 55 "QC_TypedHashMember.qpp"
xsink->raiseException("TYPEDHASHMEMBER-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// auto TypedHashMember::getValue(hash<auto> h){}
static QoreValue TypedHashMember_getValue_C10hash_auto_(QoreObject* self, QoreReflectionHashDeclMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 110 "QC_TypedHashMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    if (check_member(h, m->name.c_str(), m->th, xsink)) {
        return QoreValue();
    }

    return h->getKeyValue(m->name.c_str());
}

// bool TypedHashMember::isBuiltin(){}
static QoreValue TypedHashMember_isBuiltin(QoreObject* self, QoreReflectionHashDeclMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 67 "QC_TypedHashMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    return m->th->isSystem();
}

// bool TypedHashMember::isUser(){}
static QoreValue TypedHashMember_isUser(QoreObject* self, QoreReflectionHashDeclMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 86 "QC_TypedHashMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the typed hash ptr must be valid
    return !m->th->isSystem();
}

DLLLOCAL void preinitTypedHashMemberClass() {
    QC_TYPEDHASHMEMBER = new QoreClass("TypedHashMember", "::Qore::Reflection::TypedHashMember", QDOM_REFLECTION);
    CID_TYPEDHASHMEMBER = QC_TYPEDHASHMEMBER->getID();
    QC_TYPEDHASHMEMBER->setSystem();
}

DLLLOCAL QoreClass* initTypedHashMemberClass(QoreNamespace& ns) {
    if (!QC_TYPEDHASHMEMBER)
        preinitTypedHashMemberClass();

    // set parent class
    assert(QC_ABSTRACTMEMBER);
    QC_TYPEDHASHMEMBER->addBuiltinVirtualBaseClass(QC_ABSTRACTMEMBER);

    // TypedHashMember::constructor() {}
    QC_TYPEDHASHMEMBER->addConstructor(TypedHashMember_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // auto TypedHashMember::getValue(hash<auto> h){}
    QC_TYPEDHASHMEMBER->addMethod("getValue", (q_method_n_t)TypedHashMember_getValue_C10hash_auto_, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // bool TypedHashMember::isBuiltin(){}
    QC_TYPEDHASHMEMBER->addMethod("isBuiltin", (q_method_n_t)TypedHashMember_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool TypedHashMember::isUser(){}
    QC_TYPEDHASHMEMBER->addMethod("isUser", (q_method_n_t)TypedHashMember_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_TYPEDHASHMEMBER;
}
